#
# Get ClearCase history data.
#
#
#

# Parameters:
#
#  view=<path> The top-level view directory
#  out=<path> The path of the output file	 
#  since=<startdate> Collection start date - see ClearTool documentation
#  debug=1 Turns on debug output

$| = 1;			# Enable autoflush on stdout because of system commands.

	$logs_dir = "..\\logs";
	$vob_logs = "vobs";
	
	&parse_arguments(@ARGV);

	if (!(defined $view)) 
	{
		die "view directory must be defined";
	}

	if (!(defined $data_file)) 
	{
		die "output file must be defined";
	}
	
	if($debug == 1)
	{
		print "View is $view \n";
		print "Output file is $data_file \n";
		print "Starting point is $startdate \n";
	}

	open(DATAFILE, ">".$data_file) || die "Unable to open '$data_file' for writing.\n";

	chdir $logs_dir || die "Unable to open log directory";
	unless (opendir(DIR, $vob_logs))
	{
		mkdir($vob_logs, 0777);
		opendir(DIR, $vob_logs) || die "Unable to open vob log directory";
	}
	close (DIR);
	chdir $vob_logs;

	unless (opendir(DIR, $view))
	{
		die "Can not open view directory";
	}

	while ($file = readdir(DIR))
	{
		next if ($file =~ /^\.+$/); # skip pseudo directories
		&get_data("$view\\$file");
	}

	close (DIR);
	close (DATAFILE);

	
sub get_vob_name
{
	my ($item) = $_[0];
	
	my $cmd = "cleartool describe vob:$item 2>nul";
	
	if($debug == 1)
	{
		print $cmd, "\n";
	}
	
	open(CT, "$cmd|") or die("$cmd failed: $!\n");
	
	while(<CT>) 
	{
		chomp($_);
		my (@pieces) = split(/\n'/, $_);

		foreach $piece(@pieces)
		{
			if(grep(/versioned object base/, $piece))
			{
				$vobname = substr($piece, index($piece, "\"") + 2, length($piece) - index($piece, "\"") - 3);
			}
			break;
		}
	}
	if($debug == 1)
	{
		print "Vob is $vobname\n";
	}	
	
	close (CT);
}
sub get_data
{
	my ($item) = $_[0];
	if($debug == 1)
	{
		print "Getting history for $item\n";
	}	
	&get_vob_name($item);
	
	#If starting point for collection is not specified on the command line, use 
	#the time of last collection for this vob
		
	undef $since;
	
	if(!(defined $startdate))
	{
		if(open(COLLECTION_FILE, $vobname))
		{
			$last_collect_time = <COLLECTION_FILE>;
			close COLLECTION_FILE;

			chomp($last_collect_time);
			if($last_collect_time ne "")
			{
				if($debug == 1)
				{
					print "Last collection time is $last_collect_time\n";
				}
				$since = "-since $last_collect_time";
			}
		}
	}
	else
	{
		$since = $startdate;
	}
	
	#Record the time of this collection	
	@months = ("January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December");
	($seconds, $minutes, $hour, $monthDate, $month, $year, $weekday, $yeardate, $dls) = localtime(time);
	$year += 1900;
	$collection_time = "$monthDate-@months[$month]-$year.$hour:$minutes:$seconds";
	
	#If you want to collect different data from ClearCase, modify format string below
	my $cmd = "cleartool lshistory -recurse -fmt \"%Nd,,%u,,%o,,%m,,%En,,%Sn\\n\" $since $item 2>nul";

	if($debug == 1)
	{
		print "Executing $cmd \n";
	}
	open(CT, "$cmd|") or die("$cmd failed: $!\n");

	while(<CT>) 
	{
		chomp($_);
		# ignore the lost+found directory.
		next if(m![/\\]lost\+found[/\\]!);
		
		if($debug == 1)
		{
			print $_, "\n" ;
		}
		
		my($date, $user, $op, $kind, $path, $version) = split(/,,/, $_);
		my($day, $time) = split(/\./, $date);
		next if($op eq 'checkout');
		next if($op eq 'mkelem' && $kind eq 'branch');
		next if($op eq 'mkelem' && $kind eq 'version');
		next if(grep(/null operation kind/, $op)); #bug in ClearCase

		my $index = rindex($version, "\\");
		my $branch = substr($version, 0, $index);
		my $objectversion = substr($version, $index + 1);
		if ($objectversion eq '') 
		{
			$objectversion = 0;
		}
		
		#Format the date in the SQL date format
		my $year = substr($day, 0, 4);
		my $month = substr($day, 4, 2);
		$day = substr($day, 6);

		$date = "$year-$month-$day";

		#my($delta1, $delta2, $added, $deleted);
		my $delta1 = 0;
		my $delta2 = 0;
		my $added = 0;
		my $deleted = 0;
		
		if($op eq 'checkin' && $kind eq 'version') 
		{
			($delta1, $delta2, $added, $deleted) = diff2delta($path, $version);
			# Interpret number of lines changes in file1 to number of lines changes
			# in file2(delta1, delta2) as number of lines changes is equal to the
			# number of lines changed in file1 and if the number of lines changed
			# in file2 is greater or less then consider these lines as added or
			# deleted.
			if($delta2 > $delta1) 
			{
				$added += $delta2 - $delta1;
			} 
			elsif($delta1 > $delta2) 
			{
				$deleted += $delta1 - $delta2;
			}

		} 
		
		$outputdata = "ClearCase Data\t$date\t$user\t$op\t$kind\t$path\t$vobname\t$branch\t$objectversion\t$delta1\t$added\t$deleted";
		if($debug == 1)
		{
			print "Writing to the output file: $outputdata \n" ;
		}
		print DATAFILE "$outputdata\n";
	}
	close(CT);
	
	#Record the time of this collection
	open(COLLECTION_FILE, ">$vobname");
	print COLLECTION_FILE "$collection_time\n";
	close COLLECTION_FILE;
	
}

sub diff2delta
{
	my $delta1 = 0;
	my $delta2 = 0;
	my $added = 0;
	my $deleted = 0;
	
	my($path, $version) = @_;
	
	#ClearCase can not compare certain file types like .doc, etc.
	#Filter them out.
	my $filename = substr($path, rindex($path, "\\") + 1);
	my ($extension) = substr($filename, rindex($filename, ".") + 1);	
	if ($extension =~ m/doc/i) 
	{
		return 0,0,0,0;
	}
	
	my $cmd = qq(cleartool diff -diff_format -pred "$path\@\@$version" 2>nul);
	open(DIFF, "$cmd|")	or die("$cmd failed: $!\n");
	local($_);
	while(<DIFF>) 
	{
		if(/^\d+a\d+$/) 
		{
			$added++;
		} 
		elsif(/^\d+a(\d+),(\d+)$/) 
		{
			$added += $2 - $1;
		} 
		elsif(/^\d+d\d+$/) 
		{
			$deleted++;
		} 
		elsif(/^(\d+),(\d+)d\d+$/) 
		{
			$deleted += $2 - $1 + 1;
		} 
		elsif(/^\d+c\d+$/) 
		{
			$delta1++;
			$delta2++;
		} 
		elsif(/^(\d+),(\d+)c(\d+),(\d+)$/) 
		{
			$delta1 += $2 - $1 + 1;
			$delta2 += $4 - $3 + 1;
		}
	}
	close(DIFF);
	
	($delta1, $delta2, $added, $deleted);
}

sub parse_arguments
{
    foreach $p (@_)
	{
		if (!($p =~ /=/)) 
		{
			warn "Illegal parameter $p.  Skipping.";
			next;
		}
		local($param, $value) = split(/=/, $p, 2);
		if ($param eq "out")
		{
			$data_file=$value;
			next;
		}
		if ($param eq "view")
		{
			$view=$value;
			next;
		}
		if ($param eq "since")
		{
			$startdate="-since $value";
			next ;
		}
		if ($param eq "debug")
		{
			$debug = $value;
			next ;
		}
    }
}

1;