#
# IBM Confidential
# OCO Source Materials
# Rational ProjectConsole
# (C) Copyright IBM Corp. 2002, 2007.  All Rights Reserved.
#
# The source code for this program is not published or otherwise
# divested of its trade secrets, irrespective of what has been
# deposited with the U.S. Copyright Office.

############################################################################
#UpdateMSProjectData.pl
#
# This script determines whether a MS Project task complete and "on time" 
# or not and sets the "on time" value in the MS Project measure.  
#
############################################################################

use PjCLib;
use Win32::OLE;
require CQPerlExt;

use constant CURRENCY_SIGN => "\$";
use constant PERCENT_SIGN => "\%";
use constant CURRENCY_DEMARCATOR => "\,";
use constant LEFT_PAREN => "(";
use constant RIGHT_PAREN => ")";
use constant MINUS => "-";
use constant HOUR => " hr";
use constant HOURS => " hrs";
use constant DAY => " day";
use constant DAYS => " days";
use constant NULL_STRING => "";
use constant PROJECT_MEASURE_TABLE => "MSP_Task_Measure";
use constant PROJECT_AGGREGAGE_MEASURE_TABLE => "MSPAggregate_measure";
use constant DBID => "DBID";
use constant YES => "yes";
use constant NO => "no";
use constant BOOL_TRUE => "True";
use constant BOOL_FALSE => "False";


# Temp Fields
#   These fields hold temporary values that need to be transformed into actual values
use constant ACTUAL_COST_STR => "ActualCostStr";
use constant ACTUAL_DURATION_STR => "ActualDurationStr";
use constant ACTUAL_FINISH_DATE_STR => "actualfinish_str_1";
use constant ACTUAL_START_DATE_STR => "actualstart_str_1";
use constant ACTUAL_WORK_STR => "ActualWorkStr";
use constant ACWP_STR => "ACWPStr";
use constant BASELINE_COST_STR => "BaselineCostStr";
use constant BASELINE_DURATION_STR => "BaselineDurationStr";
use constant BASELINE_FINISH_DATE_STR => "baselinefinish_str_1";
use constant BASELINE_START_DATE_STR => "baselinestart_str_1";
use constant CURRENT_PLAN_START_DATE_STR => "curplanstart_str";
use constant CURRENT_PLAN_FINISH_DATE_STR => "curplanfinish_str";
use constant BASELINE_WORK_STR => "BaselineWorkStr";
use constant BCWP_STR => "BCWPStr";
use constant BCWS_STR => "BCWSStr";
use constant COST_STR => "CostStr";
use constant COST_VARIANCE_STR => "CostVarianceStr";
use constant FIXED_COST_STR => "FixedCostStr";
use constant PERCENT_COMPLETE_FK => "PercentCompleteStr";
use constant PERCENT_WORK_COMPLETE_FK => "PercentWorkCompleteStr";



################
# main
################

# global variables
$currentDate; 
$currentTime;
getCurrentDateTime();

print "Start Time = $currentDate $currentTime \n";

$session = logon();


# If you wish to perform the calculation for a certain day, uncomment the
# following line and set it to a day with SQL date format, e.g., "1999-05-07";
#$currentDate = "1999-06-25";

$baselineFinish = "";
$actualFinish = "";
$collInstanceLogID = "";
$finish = "";


myMain();

getCurrentDateTime();

print "End Time = $currentDate $currentTime \n";


exit;

# 
# Initialize the values of the "pseudo constants".
#
sub initialize {

    # Get physical table names.  These names will be used in the query definitions.
    $PROJECT_MEASURE_TABLE = PjCLib->getPhysicalTableDbName("MSP_Task_Measure");
    $PROJECT_AGGREGAGE_MEASURE_TABLE = PjCLib->getPhysicalTableDbName("MSPAggregate_measure");

    $COLLECTIONDATE_TABLE = PjCLib->getPhysicalTableDbName("collectiondate");
    $CALENDARDATE_TABLE = PjCLib->getPhysicalTableDbName("calendardate");
    $PROJECTDATE_TABLE = PjCLib->getPhysicalTableDbName("ProjectDate");
    $ACTUALSTARTDATE_TABLE = PjCLib->getPhysicalTableDbName("ActualStart");
    $ACTUALFINISHDATE_TABLE = PjCLib->getPhysicalTableDbName("ActualFinish");
    $BASELINESTARTDATE_TABLE = PjCLib->getPhysicalTableDbName("BaselineStart");
    $BASELINEFINISHDATE_TABLE = PjCLib->getPhysicalTableDbName("BaselineFinish");
    $CURPLANSTARTDATE_TABLE = PjCLib->getPhysicalTableDbName("curplanstart");
    $CURPLANFINISHDATE_TABLE = PjCLib->getPhysicalTableDbName("curplanfinish");
    $MSP_MILESTONE_DIM = PjCLib->getPhysicalTableDbName("msp_milestone_dim");

    # RATLC01096120: Get physical field names for dates referenced in time dimension tables.  These names will be used in the query definitions.

    $ACTUAL_START_DATE_NAME = PjCLib->getPhysicalFieldDbName("ActualStart", "ActualStart_name");
    $ACTUAL_FINISH_DATE_NAME = PjCLib->getPhysicalFieldDbName("ActualFinish", "ActualFinish_name");
    $BASELINE_START_DATE_NAME = PjCLib->getPhysicalFieldDbName("BaselineStart", "BaselineStart_name");
    $BASELINE_FINISH_DATE_NAME = PjCLib->getPhysicalFieldDbName("BaselineFinish", "BaselineFinish_name");

    # Get physical field names for the task table.  These names will be used in the query definitions.
    $TASK_NAME = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "taskname_fk");
    $TASK_ID = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "taskid_fk");
    $MEASURE = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "Number of Tasks");
    $ACTUAL_COST = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "MSPActualCost");
    $ACTUAL_DURATION = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "MSPActualDuration");
    $ACTUAL_FINISH_DATE_FK = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "actualfinish_fk");
    $ACTUAL_START_DATE_FK = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "actualstart_fk");
    $ACTUAL_WORK = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "MSPActualWork");
    $ACWP = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "ACWP");
    $BASELINE_COST = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "MSPBaselineCost");
    $BASELINE_DURATION = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "MSPBaselineDuration");
    $BASELINE_FINISH_DATE_FK = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "baselinefinish_fk");
    $BASELINE_START_DATE_FK = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "baselinestart_fk");
    $CURRENT_PLAN_START_DATE_FK = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "curplanstart_fk");
    $CURRENT_PLAN_FINISH_DATE_FK = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "curplanfinish_fk");
    $BASELINE_WORK = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "MSPBaselineWork");
    $BCWP = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "BCWP");
    $BCWS = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "BCWS");
    $COST = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "Cost_1");
    $COST_VARIANCE = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "CostVariance");
    $FIXED_COST = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "FixedCost_1");
    $PERCENT_COMPLETE = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "MSPPercentComplete");
    $PERCENT_WORK_COMPLETE = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "MSPPercentWorkComplete");
    $TASK_COMPLETE = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "MSPTaskComplete");
    $ONTIME_FK = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "ontime_fk");
    $MILESTONE_FK = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "milestone_fk");
    $COLLECTIONDATE_FK = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "collectiondate_fk");
    $COLLECTION_INSTANCE_ID = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "coll_instance_log_id");

    # Get physical field names for the aggregate table.  These names will be used in the query definitions.
    $MSP_AGGR_BP_TASK_COMPLETE = PjCLib->getPhysicalFieldDbName("MSPAggregate_measure", "MSP Aggr: BP Task Complete");
    $MSP_AGGR_CP_TASK_COMPLETE = PjCLib->getPhysicalFieldDbName("MSPAggregate_measure", "MSP Aggr: CP Task Complete");
    $MSP_AGGR_TASK_COMPLETE  = PjCLib->getPhysicalFieldDbName("MSPAggregate_measure", "MSP Aggr: Task Complete");
    $MSP_AGGR_COLLECTIONDATE_FK = PjCLib->getPhysicalFieldDbName("MSPAggregate_measure", "collectiondate_fk");
    $MSP_AGGR_CALENDARDATE_FK = PjCLib->getPhysicalFieldDbName("MSPAggregate_measure", "calendardate_fk");
    $MSP_AGGR_COLLECTION_INSTANCE_ID = PjCLib->getPhysicalFieldDbName("MSPAggregate_measure", "coll_instance_log_id");
    $MSP_AGGR_PROJECTDATE_FK = PjCLib->getPhysicalFieldDbName("MSPAggregate_measure", "projectdate_fk");

    # These fields currently not used.
    #$TRANSFORMED = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "Transformed");
    #$MILESTONE = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "Milestone");
    #$MSP_AGGR_MILESTONE_FK = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "milestone_fk");




}

sub myMain {

    my $DBID;
    my $costStr;
    my $workStr;

    PjCLib->initializePjCLibrary($session, PjCLib->BOOLEAN_TRUE);
    initialize();

    print "Update ", $PROJECT_MEASURE_TABLE, " collection dated $currentDate.  Please wait...\n";

    #
    # Get measure table results
    #
    my $mainResultSet = buildTempProjTableResultSetRaw();
    
    #
    # Loop through each table and update the column values
    #
    #print "process eachrow...\n";
 
    $count = 0;
    while ($mainResultSet->MoveNext() == $CQPerlExt::CQ_SUCCESS) {
	$DBID = $mainResultSet->GetColumnValue(1);
        $actualStartStr = $mainResultSet->GetColumnValue(2);
        $actualFinishStr = $mainResultSet->GetColumnValue(3);
	$baselineStartStr = $mainResultSet->GetColumnValue(4);
	$baselineFinishStr = $mainResultSet->GetColumnValue(5);
	$milestoneFk = $mainResultSet->GetColumnValue(6);
	$collInstanceLogID = $mainResultSet->GetColumnValue(7);
        updateProjectMeasures($DBID, 
                              $actualStartStr,
                              $actualFinishStr,
                              $baselineStartStr,
                              $baselineFinishStr,
                              );

       $count = $count + 1;
        

        ####updateTimeTableColumns($costStr);
    }
    print "Updated $count measure rows in table ", $PROJECT_MEASURE_TABLE . "\n";
    ###updateAggregateMeasures();
}

sub buildTempProjTableResultSetRaw {
    #
    # Return a result set containing the names of the time dimension tables
    # using raw SQL command
    #

    #print "buildTempProjTableResultSet started.\n";

    #RATLC01096120: Retrieve the time dimension date names vs. the foreign keys.

    $queryStr = "SELECT $PROJECT_MEASURE_TABLE.DBID, " .
        "$ACTUALSTARTDATE_TABLE.$ACTUAL_START_DATE_NAME, " .
	"$ACTUALFINISHDATE_TABLE.$ACTUAL_FINISH_DATE_NAME, " .
	"$BASELINESTARTDATE_TABLE.$BASELINE_START_DATE_NAME, " .
	"$BASELINEFINISHDATE_TABLE.$BASELINE_FINISH_DATE_NAME, " .
	"$PROJECT_MEASURE_TABLE.$MILESTONE_FK, " .
	"$PROJECT_MEASURE_TABLE.$COLLECTION_INSTANCE_ID " .
	"FROM ((($PROJECT_MEASURE_TABLE INNER JOIN " . 
	"$ACTUALSTARTDATE_TABLE ON $PROJECT_MEASURE_TABLE.$ACTUAL_START_DATE_FK = $ACTUALSTARTDATE_TABLE.dbid) " .
	"INNER JOIN $BASELINEFINISHDATE_TABLE ON $PROJECT_MEASURE_TABLE.$BASELINE_FINISH_DATE_FK = $BASELINEFINISHDATE_TABLE.dbid) " .
	"INNER JOIN $BASELINESTARTDATE_TABLE ON $PROJECT_MEASURE_TABLE.$BASELINE_START_DATE_FK = $BASELINESTARTDATE_TABLE.dbid) " .
  	"INNER JOIN $ACTUALFINISHDATE_TABLE ON $PROJECT_MEASURE_TABLE.$ACTUAL_FINISH_DATE_FK = $ACTUALFINISHDATE_TABLE.dbid " .
	"WHERE $PROJECT_MEASURE_TABLE.$COLLECTIONDATE_FK = (SELECT DBID FROM " .
	"collectiondate WHERE collectiondate_name = '$currentDate')";

    #print "Excute query $queryStr...\n";

    my $projResultSet = $session->BuildSQLQuery($queryStr);
    $projResultSet->Execute();

    #print "buildTempProjTableResultSet completed.\n";
    return $projResultSet;
}


sub replace {
    my ($text, $toReplace, $replaceWith,) = @_;

    #print "replace $text, $toReplace, $replaceWith\n";
    # replace all occurances

    my $notDone = 1;
    while($notDone) {
        my $index = index($text, $toReplace);
        if ($index >= 0) {
            substr($text, $index, length($toReplace), $replaceWith);
        } 
        else {
            $notDone = 0;
        }
    }
    #print "transformed to $text\n";
    return $text;
}


sub updateProjectMeasures {
    my ($DBID, 
        $actualStartStr,
        $actualFinishStr,
        $baselineStartStr,
        $baselineFinishStr,
        ) = @_;

    #printf("process $count : DBID $DBID\n");

    #RATLC01096120: Dates no longer need to be converted since the date names are retrieved from time
    #dimension tables, which means that the date is already in the right format.

    $actualStartDateStr = $actualStartStr;
    #printf("actualStartStr    => $actualStartStr transformed to '$actualStartDateStr' \n");

    # set the global var
    $actualFinish = $actualFinishStr;
    #printf("actualFinishStr   => $actualFinishStr transformed to '$actualFinish' \n");
    # Determine task complete measure
    my $taskComplete = "0";
    if (length($actualFinish) != 0) {
       $taskComplete = "1";
    }
    #printf("$DBID taskComplete  = $taskComplete; ");


    # Determine if task is on time
    my $onTime = isOnTime($actualFinishStr, $baselineFinishStr);
    #printf("onTime = $onTime\n");


    #
    # Get the entity for this record.  Note that updates must be made
    # using an entity object.
    #
    my $entity = $session->GetEntityByDbId($PROJECT_MEASURE_TABLE, $DBID);


    #
    # Update the record
    #
    #print "Update record $DBID\n";
    $session->EditEntity($entity,"modify");
    $entity->SetFieldValue($ONTIME_FK, $onTime);
    $entity->SetFieldValue($TASK_COMPLETE, $taskComplete);
    $retVal = $entity->Validate();
    #printf("entity->Validate() return value = $retVal\n");
    $entity->Commit();
    #printf("entity->Commit() return value = $retVal\n");

    #printf("Measure Updated\n");


}

  
sub isOnTime{
    my ($actualFinishStr, $baselineFinishStr) = @_;

    #printf("isOnTime() executing...\n");
    #printf("  actualFinishStr  = $actualFinishStr\n");
    #printf("  baselineFinishStr = $baselineFinishStr\n");
    if (length($baselineFinishStr) == 0) {
       # baseline finish date is not set so return True
       return BOOL_TRUE;
    } else {
       if (length($actualFinishStr) == 0) {
          # actual start is not set, so check if it's late.  If collectiondate is 
          # after baseline finish, then it's late
          if ($currentDate gt $baselineFinishStr) {
             return BOOL_FALSE;
          } else {
             return BOOL_TRUE;
          }
       } else {
          if ($actualFinishStr gt $baselineFinishStr) {
             return BOOL_FALSE;
          }
       }
    }
    return BOOL_TRUE;
}

sub transformToHours{
    my ($value, $fieldName) = @_;


    my $actualHours = replace($value, HOURS, NULL_STRING);
    $actualHours = replace($actualHours, HOUR, NULL_STRING);
    my $actualHoursInt = 0 + $actualHours;
    my $actualHoursStr = sprintf("%.d", $actualHoursInt);   

    #print " transformed day value $value to $actualHoursStr\n";
    return $actualHoursStr;
}

sub transformToDays{
    my ($value, $fieldName) = @_;

    my $actualDays = replace($value, DAYS, NULL_STRING);
    $actualDays = replace($actualDays, DAY, NULL_STRING);

    my $actualDaysInt = 0 + $actualDays;
    my $actualDaysStr = sprintf("%.d", $actualDaysInt);

    #print " transformed day value $value to $actualDaysStr\n";

    return $actualDaysStr;
}

sub transformPercent {
    my ($percentName, $percentValue) = @_;

    if (length($percentValue) == 0) {
        return 0;
    }

    # Strip off percent sign
    my $percentValueStr = replace($percentValue, PERCENT_SIGN, NULL_STRING);

    my $percentValueInt = $percentValueStr;
    #print "transformPercent(): ", "'; $percentName  ", $percentValue , " tranformed to => ", $percentValueInt, "\n";

    return sprintf($percentValueStr);
}

sub transformCurrencyToCents {
    my ($amountName, $amountStr) = @_;

    # Strip off dollar sign
    my $amountIntStr = replace($amountStr, CURRENCY_SIGN, NULL_STRING);

    # Need to strip out any currency demarcator: e.g., "1,234.00" => "1234.00".  Otherwise,
    # we can't convert it to an integer.
    $amountIntStr = replace($amountIntStr, CURRENCY_DEMARCATOR, NULL_STRING);

    # Need to convert (), which means it is a negative amount to "-" minus sign.
    $amountIntStr = replace($amountIntStr, LEFT_PAREN, MINUS);
    $amountIntStr = replace($amountIntStr, RIGHT_PAREN, NULL_STRING);


    my $amountInt = int($amountIntStr);
    #print $DBID, ": ", "'; $amountName value ",$amountStr , "tranformed to => ", $amountInt, "\n";

    return $amountInt;
}

sub updateTaskNameDim {
    my ($taskNameStr) = @_;

    print "Update taskname '", $taskNameStr, "' in the taskname_dim table\n";
    my $query = "select task_name from taskname_dim where dbid <> 0 and task_name = '" . $taskNameStr . "'";
    print "Query => ", $query , "\n";
    
    my $resultSet = $session->BuildSQLQuery($query);
    
    $resultSet->Execute();
    $status = $resultSet->MoveNext();
    #print "CQ_SUCCESS       =>  ", $CQPerlExt::CQ_SUCCESS , "\n";
    #print "CQ_NO_DATA_FOUND =>  ", $CQPerlExt::CQ_NO_DATA_FOUND , "\n";
    #print "Status =>  ", $status , "\n";
    if ($status == $CQPerlExt::CQ_NO_DATA_FOUND) {
        my $taskNameEntity = $session->BuildEntity("taskname_dim");
        $taskNameEntity->SetFieldValue("task_name", $taskNameStr);
        $taskNameEntity->Validate();
        $taskNameEntity->Commit();
        #printf("taskname_dim table updated\n");
    }

}

# get date and set the global variables $currentDate and $currentTime
sub getCurrentDateTime {
    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);

    $mon = $mon + 1;
    $year = $year + 1900;

    if ($mday<10) {$mday="0$mday"; }  # add leading 0
    if ($mon<10) {$mon="0$mon"; }     # add leading 0

    # set the global variable $currentDate
    $currentDate="$year-$mon-$mday";

    # set the global variable $currentTime
    $currentTime = $hour . ":" . $min . ":" . $sec;

}

# MS Project date format is something like "Fri 2/19/99";
# This will convert the date to "1999-02-19";
sub convertMSProjectDate {
    ($_) = @_;

    if ($_ == "NA") {
      return "";
    }
    # remove spaces and useless day
    #$_ = $temp;
    s/^\s+//;  # kill leading spaces
    s/^(?:Sun|Mon|Tue|Wed|Thu|Fri|Sat)[a-z]*,?\s*//i; # kill useless weekday
    
    #printf("converting ". $_ . "\n");
    @array = split(/\//, $_);
    my ($mon,$day,$yr) = @array;
    if ($yr < 100) {
      $yr += 1900;
    }
    my $newDate = sprintf("%04d-%02d-%02d", $yr, $mon, $day);  # add leading 0 on month and day

    
    return $newDate;
}

sub zeroPad {
    my ($n,$Str) = @_;

    #
    # Simple routine to add leading zeros to a string
    #
    while (length($Str) < $n) { $Str = "0".$Str; }
    return $Str;
}



sub logon {

    #
    # Use the internal DashboardSystem login to get access to the Dashboard database
    #
    my $session = Win32::OLE->new('CLEARQUEST.SESSION');
    my $session = CQPerlExt::CQSession_Build();  
    $session->UserLogon("dashboardsystem", "dashboardsystem", "tstd1", "Dashboard");
    return $session;
}


#
# Return the DBID of the specified field value from specified table.
#
sub getDBID {
    my ($tableName, $fieldName, $fieldValue) = @_;
    my $DBID = "0";

    #if (length($fieldValue) == 0) {
    #    return $DBID";
    #}

    #print "buildTempProjTableResultSet started.\n";
    my $queryStr = "SELECT DBID FROM $tableName" .
                " WHERE $fieldName = '$fieldValue'";
    #print "getDBID(): Execute query $queryStr\n";

    my $resultSet = $session->BuildSQLQuery($queryStr);
    $resultSet->Execute();

    while ($resultSet->MoveNext() == $CQPerlExt::CQ_SUCCESS) {
	$DBID = $resultSet->GetColumnValue(1);
    }
    return $DBID;
}

############################################################################################
#
# The constants and subroutines below are not currently not used but are kept as examples.
#


# NOT USED - keep for example
sub updateAggregateMeasures {
#select baselinefinish_fk from msptask_measure
#where baselinefinish_fk > 0
#and coll_instance_log_id_4 = 33618507
#group by baselinefinish_fk

    #
    # Return a result set containing the names of the time dimension tables
    # using raw SQL command
    #

    #print "updateAggregateMeasures():  started.\n";
    $queryStr = "SELECT " . $BASELINE_FINISH_DATE_FK . 
                " FROM " . $PROJECT_MEASURE_TABLE .
                " WHERE " . $BASELINE_FINISH_DATE_FK . " > 0 AND " .
                $COLLECTION_INSTANCE_ID . " = $collInstanceLogID " .
                "GROUP BY " . $BASELINE_FINISH_DATE_FK;
    #print "Excute query $queryStr...\n";

    my $projResultSet = $session->BuildSQLQuery($queryStr);
    $projResultSet->Execute();

    my $count = 0;
    while ($projResultSet->MoveNext() == $CQPerlExt::CQ_SUCCESS) {
       $count = $count + 1;
       $baselineFinishDBID = $resultSet->GetColumnValue(1);
       #print "   baselineFinishDBID = $baselineFinishDBID\n";
    }
    #printf "Query returned $count rows in table.\n";

}



# NOT USED - keep for example
sub buildTempProjTableResultSet {

    #
    # Return a result set containing the names of the time dimension tables using CQ API
    # WARNING: This doesn't work for some unknown reason!
    #
    
    print "buildTempProjTableResultSet started.\n";

    ####my $timeResultSet = $session->BuildSQLQuery("select name from actual where dbid <> 0 and table_type = '2' order by name");

    # build a querydef for the measure table
    my $queryDef = $session->BuildQuery($PROJECT_MEASURE_TABLE);
    $queryDef->BuildField(DBID);
    $queryDef->BuildField(ACTUAL_COST_STR);
    $queryDef->BuildField(ACTUAL_DURATION_STR);
    $queryDef->BuildField(ACTUAL_WORK_STR);
    $queryDef->BuildField(ACWP_STR);
    $queryDef->BuildField(BASELINE_COST_STR);
    $queryDef->BuildField(BASELINE_DURATION_STR);
    $queryDef->BuildField(BASELINE_WORK_STR);
    $queryDef->BuildField(BCWP_STR);
    $queryDef->BuildField(BCWS_STR);

    #print "Build operator...\n";
    $filterOperator = $queryDef->BuildFilterOperator($CQPerlExt::CQ_BOOL_OP_AND);
    #print "set filter: ", $COLLECTIONDATE_FK, " == $currentDate\n";
    #print "comp op eq == $CQPerlExt::CQ_COMP_OP_EQ\n";
    #$filterOperator->BuildFilter($COLLECTIONDATE_FK, $CQPerlExt::CQ_COMP_OP_EQ, $currentDate);
    $filterOperator->BuildFilter("taskid_fk", $CQPerlExt::CQ_COMP_OP_EQ, "2");

    #print "BuildResultSet...\n";
    my $projResultSet = $session->BuildResultSet($queryDef);
    
    $projResultSet->Execute();

    #print "buildTempProjTableResultSet completed.\n";
    return $projResultSet;
}

# NOT USED - keep for example
sub updateAggregateMeasuresNotUsed {

    my ($DBID,
        $currentDate,
        $baselineFinish,
        $currentPlanFinish,
        $actualFinish,
        $milestoneFk
        ) = @_;

    my $currentDateDBID = getDBID($COLLECTIONDATE_TABLE, $COLLECTIONDATE_TABLE . "_name", $currentDate);
    my $baselineFinishDBID = getDBID($BASELINEFINISHDATE_TABLE, $BASELINEFINISHDATE_TABLE . "_name", $baselineFinish);
    my $currentPlanFinishDBID = getDBID($CURPLANFINISHDATE_TABLE, $CURPLANFINISHDATE_TABLE . "_name", $currentPlanFinish);
    my $actualFinishDBID = getDBID($ACTUALFINISHDATE_TABLE, $ACTUALFINISHDATE_TABLE . "_name", $actualFinish);
    printf("updateAggregateMeasures():\n");
    printf("    DBID           = $DBID\n");
    printf("    currentDate    = $currentDateDBID, $currentDate\n");

    printf("    baselineFinish = $baselineFinishDBID, $baselineFinish\n");
    printf("    currentPlanFinish = $currentPlanFinishDBID, $currentPlanFinish\n");
    printf("    actualFinish      = $actualFinishDBID, $actualFinish\n");
    printf("    milestoneFk       = $milestoneFk\n");

    my $numBaselineTaskComplete = getAggregateValue(MSPTask_Measure,
                                                    $currentDate,
                                                    );

    #
    # Return a result set containing the names of the time dimension tables
    # using raw SQL command
    #

    #print "buildTempProjTableResultSet started.\n";
    $queryStr = "SELECT " . DBID . "," .
                $MSP_AGGR_BP_TASK_COMPLETE . "," .
                $MSP_AGGR_CP_TASK_COMPLETE . "," .
                $MSP_AGGR_TASK_COMPLETE . "," .
                $MSP_AGGR_CALENDARDATE_FK . "," .
                $MSP_AGGR_MILESTONE_FK .
                " FROM " . $PROJECT_AGGREGAGE_MEASURE_TABLE .
                " WHERE " . $MSP_AGGR_COLLECTIONDATE_FK . " = $currentDateDBID";
    print "Excute query $queryStr...\n";

    my $projResultSet = $session->BuildSQLQuery($queryStr);
    $projResultSet->Execute();

    #print "buildTempProjTableResultSet completed.\n";
    #print "process eachrow...\n";
    my $count = 0;
    while ($projResultSet->MoveNext() == $CQPerlExt::CQ_SUCCESS) {
       $count = $count + 1;
    }
    printf ("Query returned $count rows in the aggregate table.\n");

    
}

#
# 
# NOT USED - Save as example
#
sub getAggregateValue {
    my (
        $tableName,
        $measureField,
        $dateField,
        $dateValue,
        $collInstanceLogField,
        $collInstanceLogValue
       ) = @_;


    my $queryStr = "SELECT SUM($measureField) FROM  " . $tableName . 
                   " WHERE $dateField > 0 AND $dateField <= $dateValue AND " .
                   " $collInstanceLogField = $collInstanceLogValue";
    #print "getAggregateValue(): Execute query $queryStr...\n";

    my $resultSet = $session->BuildSQLQuery($queryStr);
    $resultSet->Execute();

    #print " query completed.\n";

    if ($resultSet->MoveNext() == $CQPerlExt::CQ_SUCCESS) {
       $result = $resultSet->GetColumnValue(1);
    }
    #print "    result = $result\n";
    return $result;
}

#
# delete all rows in aggregate table!!
# 
# NOT USED - Save as example
#
sub updateAggregateTable {
    my ($tableName,
        $calendarDate,
        $measureFieldName,
        $measureValue,
       ) = @_;

    MY $queryStr = "SELECT " . DBID .
                " FROM $tableName  WHERE " . 
                $MSP_AGGR_CALENDARDATE_FK . " = $calendarDate ";
    #print "updateAggregateTable(): Excute query $queryStr...\n";

    my $resultSet = $session->BuildSQLQuery($queryStr);
    $resultSet->Execute();

    #print " query completed.\n";
    my $entity;

    if ($resultSet->MoveNext() == $CQPerlExt::CQ_SUCCESS) {
       # do a MODIFY
       $DBID = $resultSet->GetColumnValue(1);

       print "MODIFY aggregate record $DBID\n";
       $entity = $session->GetEntityByDbId($tableName, $DBID);
       $session->EditEntity($entity,"modify");
    }
    else {
       # do an INSERT
       print "INSERT aggregate record\n";
       $entity = $session->BuildEntity($tableName);
    }
    $entity->SetFieldValue($MSP_AGGR_CALENDARDATE_FK,$calendarDate);
    $entity->SetFieldValue($measureFieldName,$measureValue);

    $retVal = $entity->Validate();
    #printf("entity->Validate() return value = $retVal\n");
    $entity->Commit();
    #printf("entity->Commit() return value = $retVal\n");

    #printf("Aggregate Measure Updated\n");
}
